/**
 *
 * @file
 * Copyright &copy; Audi Electronics Venture GmbH. All rights reserved
 *
 */

/*
 * This file depends on Qt which is licensed under LGPLv3.
 * See ADTF_DIR/3rdparty/qt5 and doc/license for detailed information.
 */

#pragma once
#include <QtWidgets>

#include <adtffiltersdk/adtf_filtersdk.h>
#include <adtfmediadescription/codec_sample_streamer.h>
// always include filtersdk, systemsdk or streaming3 sdk BEFORE adtfui!!
#include <adtf_ui.h>

using namespace adtf::ucom;
using namespace adtf::streaming;
using namespace adtf::filter;
using namespace adtf::mediadescription;
using namespace adtf::ui;

// this is a helper that handles a single input that creates and updates
// the corresponding subtree
class cInput
{
public:
    cInput(decoding_sample_reader<cSingleSampleReader>* pReader);
    QTreeWidgetItem* GetRootItem();
    void Update();

private:
    void IndicateNoDescription();
    tResult TypeChanged(const iobject_ptr<const IStreamType>& pType);
    void AddElement(const QString& strName, QTreeWidgetItem* pParent);
    void ClearItems();

    decoding_sample_reader<cSingleSampleReader>* m_pReader = nullptr;
    object_ptr<const ISample> m_pLastSample;
    QTreeWidgetItem* m_pRootItem = nullptr;
    std::vector<QTreeWidgetItem*> m_oElementItems;
};

class cQtMediaDescFilter : public adtf::ui::cQtUIDynamicFilter
{
    public:
        ADTF_CLASS_ID_NAME(cQtMediaDescFilter,
                           "tutorial_filter_ui.ui_filter.adtf.cid",
                           "DDL UI Filter");

        ADTF_CLASS_DEPENDENCIES(REQUIRE_INTERFACE(adtf::ui::IQtXSystem));

    public:
        cQtMediaDescFilter();

        tResult RequestDynamicInputPin(const char* strName,
                                       const adtf::ucom::ant::iobject_ptr<const adtf::streaming::ant::IStreamType>& pType) override;

    protected: // Implement cBaseQtFilter
        QWidget* CreateView() override;
        void ReleaseView() override;
        tResult OnTimer() override;

    private:
        std::vector<std::unique_ptr<cInput>> m_oInputs;
        QTreeWidget* m_pTree = nullptr;
};
